/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.core;

import java.io.InputStream;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

public abstract class AbstractTreeStorage implements ITreeStorage {
    private IStorageDeltaListener deltaTarget;

    public AbstractTreeStorage() {
    }

    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#setDeltaTarget(com.unitesk.requality.core.IStorageDeltaListener)
     */
    @Override
    public void setDeltaTarget(IStorageDeltaListener deltaTarget) {
        this.deltaTarget = deltaTarget;
    }

    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getDeltaTarget()
     */
    @Override
    public IStorageDeltaListener getDeltaTarget() {
        return deltaTarget;
    }

    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#createNode(java.util.UUID, java.util.UUID, java.lang.String, java.lang.String)
     */
    @Override
    public void createNode(UUID parentNode, UUID node, String type, String id) { 
        if(parentNode==null) {
            type = "DBRoot";
        }
        Set<Attribute> attr = new HashSet<Attribute>();
        attr.add(new Attribute(AttributeType.STRING, TreeNode.ATTR_TYPE, type));
        createNode(parentNode, node, id, attr);
    }

    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getNodeType(java.util.UUID)
     */
    @Override
    public String getNodeType(UUID uuid) {
        for(Attribute a : getAttributes(uuid)) {
            if(a.getKey().equals(TreeNode.ATTR_TYPE)) {
                return (String) a.getValue();
            }
        }
        return null;
    }

    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getRootNode()
     */
    @Override
    abstract public UUID getRootNode();
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getParent(java.util.UUID)
     */
    @Override
    abstract public UUID getParent(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getChildren(java.util.UUID)
     */
    @Override
    abstract public Collection<UUID> getChildren(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#createNode(java.util.UUID, java.util.UUID, java.lang.String, java.util.Collection)
     */
    @Override
    abstract public void createNode(UUID parentUUId, UUID uuid, String id, Collection<Attribute> attrs);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#remove(java.util.UUID)
     */
    @Override
    abstract public void remove(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getNodeId(java.util.UUID)
     */
    @Override
    abstract public String getNodeId(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#changeNodeId(java.util.UUID, java.lang.String)
     */
    @Override
    abstract public void changeNodeId(UUID uuid, String id);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getAttributes(java.util.UUID)
     */
    @Override
    abstract public Collection<Attribute> getAttributes(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#saveAttributes(java.util.UUID, java.util.Collection)
     */
    @Override
    abstract public void saveAttributes(UUID uuid, Collection<Attribute> attributes);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#moveNodeTo(java.util.UUID, java.util.UUID)
     */
    @Override
    abstract public void moveNodeTo(UUID uuid, UUID parent);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#canLoad(java.util.UUID)
     */
    @Override
    abstract public boolean canLoad(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#isResourceExists(java.util.UUID, java.lang.String)
     */
    @Override
    abstract public boolean isResourceExists(UUID uuid, String resName);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getNodeResourcePath(java.util.UUID)
     */
    @Override
    abstract public String getNodeResourcePath(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getResourceContents(java.util.UUID, java.lang.String)
     */
    @Override
    abstract public InputStream getResourceContents(UUID uuid, String resName);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#setResourceContents(java.util.UUID, java.lang.String, java.io.InputStream)
     */
    @Override
    abstract public void setResourceContents(UUID uuid, String resName, InputStream in, Object data);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#listResources(java.util.UUID)
     */
    @Override
    abstract public String[] listResources(UUID uuid);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#deleteResource(java.util.UUID, java.lang.String)
     */
    @Override
    abstract public void deleteResource(UUID uuid, String resName);

    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#createStorage(java.lang.String)
     */
    @Override
    abstract public void createStorage(String name);
    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#deleteStorage()
     */
    @Override
    abstract public void deleteStorage();

    /* (non-Javadoc)
     * @see com.unitesk.requality.core.ITreeStorage#getName()
     */
    @Override
    abstract public String getName();
}
