package com.unitesk.requality.core;

import java.util.Comparator;
import java.util.List;
import java.util.UUID;

import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class TreeNodeComparator {
    private static Comparator<TreeNode> comparator;
    public static Comparator<TreeNode> getComparator(){
        if(comparator == null)
            comparator = new Comparator<TreeNode>() {
                @Override
                public int compare( TreeNode n1, TreeNode n2 )
                {
                    if (n1.getType().compareTo(n2.getType()) != 0) {
                        return n1.getType().compareTo(n2.getType());
                    }
                    if(    n1 instanceof Location 
                        && n2 instanceof Location 
                        && n1.getParent().getQualifiedId().equals( n2.getParent().getQualifiedId()) 
                        && n1.getParent() instanceof Document 
                      ) 
                    {
                        String id1 = n1.getId(), id2 = n2.getId();
                        if (id1.equals(id2)) 
                        {
                            return 0;
                        }
                        List<UUID> locs = ((Document) n1.getParent()).getLocations();
                        int i1 = -1, i2 = -1;
                        for (int i = 0; i < locs.size(); i++) {
                            String id = locs.get(i).toString();
                            if (id.equals(id1)) {
                                i1 = i;
                            }
                            if (id.equals(id2)) {
                                i2 = i;
                            }
                            if (i1 >= 0 && i2 >= 0) {
                                break;
                            }
                        }
                        return i1 - i2;
                    }
                    int k = n1.getIndex().compareTo( n2.getIndex() );
                    //Add name handling
                    if(k == 0){
                        String s1 = n1.getId(), s2=n2.getId();
                        if (n1.getType().equals(Requirement.TYPE_NAME)){
                            String t1 = ((Requirement)n1).getName(),
                                   t2 = ((Requirement)n2).getName();
                            if(t1 != null) s1 = t1;
                            if(t2 != null) s2 = t2;
                        }
                        return s1.compareTo( s2 );
                    }
                    return k;
                }
            };
        return comparator;
    }
}
