package com.unitesk.requality.eclipse.handlers;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.tools.Index;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateCommentHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> tnlist = RequalityCNF.getSelectedNodes();
        if(tnlist == null || (tnlist.size() > 1))
            return null;
        TreeNode tn = tnlist.get(0), node = null;
        if(!tn.getTreeDB().getTreeLogic().canBeChildOf(Comment.TYPE_NAME, tn)){
            if(tn.getType().equals(Comment.TYPE_NAME)){
                node = tn;
                tn = tn.getParent();
            }else{
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Add Element",
                        "Element of type '" + Comment.TYPE_NAME
                        + "' cannot be created in " + tn.getUserFriendlyName());
                return null;
            }
        }
        String id = RequalityCLI.getNextId(tn, Comment.TYPE_NAME, Comment.TYPE_NAME + " ");
        tn.getTreeDB().startTransaction("Create Comment");
        Index ind = Index.getNextIndex(Comment.TYPE_NAME, tn, node, false);
        Comment tc = (Comment) tn.createChild(id,
                Comment.TYPE_NAME);
        tc.setIndex(ind);
        //add default author attribute to comment
        tc.putAttribute(new Attribute(AttributeType.STRING,
                Comment.ATTR_AUTHOR, System.getProperty("user.name", "")));
        tc.saveAttributes();
        tn.getTreeDB().commit();
        Activator.getDefault().setActiveNode(tc, Comment.TYPE_NAME);
        return null;
    }

    @Override
    public boolean isEnabled() {
        List<TreeNode> nodelist = RequalityCNF.getSelectedNodes();
        if (nodelist == null || nodelist.size() > 1)
            return false;
        if (nodelist.get(0).getType().equals(Comment.TYPE_NAME))
            return true;
        return nodelist.get(0).getTreeDB().getTreeLogic().canBeChildOf("Comment", nodelist.get(0));
    }

}
