/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.handlers.requirement;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateRequirementHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        String id = RequalityCLI.getNextId(nodes.get(0), Requirement.TYPE_NAME, null);
        TreeDB db = nodes.get(0).getTreeDB();
        try {
            if(!nodes.get(0).getTreeDB().getTreeLogic().canBeCreatedInExc(Requirement.TYPE_NAME, id, nodes.get(0))){
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Create Element",
                        "Element of type '" + Requirement.TYPE_NAME 
                        + "' cannot be created in " + nodes.get(0).getUserFriendlyName());
                return null;
            }
        } catch (PastingIntoLeafException e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Create Element",
                    "Can't add "+Requirement.TYPE_NAME+" to "+CoreUtils.splitWords(nodes.get(0).getType())+" with "
                    + "Test Purpose child");
            return null;
        } catch (SameIdExistsInChildren e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Create Element",
                    "Element with same id exists in " + 
                    nodes.get(0).getUserFriendlyName());
            return null;
        } catch (IdEmpty e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Create Element",
                    "Element id is empty");
            return null;
        } catch (IdContainsWrongChars e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Create Element",
                    "Element id contains wrong character - " 
                    + e.getWrongCharacter());
            return null;
        }
        db.startTransaction("Create Requirement");
        nodes.get(0).createChild(id, Requirement.TYPE_NAME);
        nodes.get(0).saveAttributes();
        db.commit();
        return null;
    }

    @Override
    public boolean isEnabled() {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if (nodes == null) {
            return false;
        }
        if (!nodes.get(0).getType().equals(Requirement.TYPE_NAME)) {
            return false;
        }
        if (countNotRequirementChildren(nodes.get(0)) > 0) {
            return false;
        }
        return true;
    }

    private int countNotRequirementChildren(TreeNode node) {
        int count = 0;
        for (int i = 0; i < node.sizeChildren(); i++) {
            if (node.getChild(i).getType().equals(Requirement.TYPE_NAME)) {
                continue;
            }
            if (node.getChild(i).getType().equals(Comment.TYPE_NAME)) {
                continue;
            }
            count++;
        }
        return count;
    }
}
