/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.TreeLogic;

public class CreateFileDialog extends Dialog {
	private static String makeZeroString(int zeroCount) {
        StringBuilder sb = new StringBuilder(zeroCount);
        for (int i = 0; i < zeroCount; i++)
            sb.append("0");
        return sb.toString();
    }

	public static String incLastNumberInString(String str) {
        String regex = "\\d*";
        String tmp = str;

        while (!tmp.matches(regex))
            tmp = tmp.substring(1);
        String fixture = str.substring(0, str.length() - tmp.length());
        String tail = String.valueOf(Integer.parseInt(tmp) + 1);

        if (tail.length() < tmp.length()) {
            tail = makeZeroString(tmp.length() - tail.length()) + tail;
        }
        return fixture + tail;
    }
	private static String getNumberStringFromId(String in) {
        Pattern pattern = Pattern.compile("(.*\\D|)(\\d*)$");
        Matcher m = pattern.matcher(in);
        if (m.matches()) {
            return m.group(m.groupCount());
        } else {
            return null;
        }
    }
	private String getNextId(IFolder f) throws CoreException{
    	String newId = "000";
        String fullNewId = "000";
        boolean found = false;

        for (int i = 0; i < f.members().length; i++) {
        	IResource ir = f.members()[i];
        	if(!(ir instanceof IFile))
        		continue;
        	IFile file = (IFile) ir;
        			
            String number = getNumberStringFromId(file.getName());
            if (number == null) {
                continue;
            }
            try {
                if (Integer.parseInt(number) >= Integer.parseInt(newId)) {
                    fullNewId = file.getName();
                    newId = number;
                    found = true;
                }
            } catch (NumberFormatException e) {
            }
        }

        if (!found) {
            fullNewId = f.getName()+ "-" + fullNewId;
        }

        do {
            fullNewId = incLastNumberInString(fullNewId);
        } while (f.findMember(fullNewId+extension) != null);
        return fullNewId;
	}
	
    private final class FolderSelectionListener implements
            ISelectionChangedListener {
    	CreateFileDialog cf;
    	public FolderSelectionListener(CreateFileDialog cfd){
    		cf = cfd;
    	}
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            TreeSelection ts = (TreeSelection) event.getSelection();
            if (ts.getFirstElement() instanceof IFolder) {
            	if(!ts.getFirstElement().equals(selectedNode)){
            		select(ts.getFirstElement());
            		try {
						fileName.setText(cf.getNextId(selectedNode));
					} catch (CoreException e) {
					}
            	}
            } 
            updateUI();
        }
    }

    private MouseListener mouseList = new MouseListener() {
        public void mouseDoubleClick(MouseEvent e) {
            TreeItem item = nodeTree.getTree().getItem(new Point(e.x, e.y));
            if (item != null && canSelectNode())
                okPressed();
            //buttonBar.setEnabled(canSelectNode());
        }

        public void mouseDown(MouseEvent e) {
        }

        public void mouseUp(MouseEvent e) {
        }
    };

    private KeyListener keyList = new KeyListener() {
        public void keyPressed(KeyEvent e) {
            if (e.keyCode == ' ') {
                TreeItem[] items = nodeTree.getTree().getSelection();
                if (items.length > 0) {
                    TreeItem item = items[0];
                    if (item.getExpanded() == false) {
                        item.setExpanded(true);
                        Event e1 = new Event();
                        e1.item = item;
                        nodeTree.getTree().notifyListeners(SWT.Expand, e1);
                    } else
                        item.setExpanded(false);
                }
            }
        }

        public void keyReleased(KeyEvent e) {
        }
    };

    private static int TREE_WIDTH = 250;
    private static int TREE_HEIGHT = 300;

    private boolean canSelectRoot = false;

    private Button createButton = null;
    protected IFolder selectedNode = null;
    protected String newNodeType = null;

    private TreeViewer nodeTree = null;
    protected Composite header = null;
    protected String childtype = null;
    protected Text fileName;
    protected String lastCorrectId = null;
    
    public void setTypeOfCreatedChilden(String t){
        childtype = t;
    }
    protected String defaultIdPrefix(){ return null; } 
    

    @Override
    public int open() {
        return super.open();
    }

    @Override
    protected void buttonPressed(int buttonId) {
        super.buttonPressed(buttonId);
    }

    public boolean canCreateNode() {
        return selectedNode != null;
    }

    public boolean canSelect() {
        return selectedNode != null && canSelectNode();
    }

    private void updateUI() {
        Button ok = getButton(IDialogConstants.OK_ID);
        if (ok == null || ok.isDisposed()) {
            return;
        }
        if (createButton != null) {
            createButton.setEnabled(canCreateNode());
        }
        ok.setEnabled(canSelect());
    }

    public boolean canSelectNode() {
        if (selectedNode == null)
            return false;
        if (!(selectedNode instanceof IFolder))
        	return false;
        return true;
    }
    
    public IFolder getResult() {
        if (selectedNode instanceof IFolder) {
            return selectedNode;
        }
        return null;
    }
    public String getResultFileName() {
        return lastCorrectId;
    }
    private String extension;

	private String title;


    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = createClientContainer(parent, 1);
        
        header = new Composite(container, SWT.NONE);
        header.setLayout(new GridLayout(2, false));
        header.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        new Label(header, SWT.NONE).setText("File Name:");
        fileName = new Text(header, SWT.FILL);
        fileName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        final ControlDecoration fileNameDecorator = new ControlDecoration(fileName,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        fileNameDecorator.setImage(fieldDecoration.getImage());
        
        fileName.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
            	String s = fileName.getText(), err = null;
            	if(s == null || selectedNode == null || selectedNode.findMember(s + extension) != null){
                	err = "File with name "+fileName.getText()+" already exists";
            	}
            	//it is possible for this method to be moved to Utils in future
            	try {
					if(!TreeLogic.isIdValidExc(s))
						err = "File name "+s+" is not valid";
				} catch (IdEmpty e1) {
					err = "File name is empty";
				} catch (IdContainsWrongChars e1) {
					err = "File name "+s+" contains wrong character - '"+e1.getWrongCharacter()+"'";
				}
                if(getButton(Dialog.OK)!=null) {
                    if (err != null) {
                        getButton(Dialog.OK).setEnabled(false);
                    } else {
                        getButton(Dialog.OK).setEnabled(true);
                    }
                }
                if (err != null) {
                	fileNameDecorator.setDescriptionText(err);
                	fileNameDecorator.show();
                    return;
                }
                lastCorrectId = s + extension;
                fileNameDecorator.hide();
            }
        });
        
        
        Label commentLabel = new Label(container, SWT.NONE);
        String labelString = "Select a folder:";
        commentLabel.setText(labelString);
        commentLabel.setLayoutData(new GridData(GridData.BEGINNING,
                GridData.CENTER, true, true));
        nodeTree = new TreeViewer(container);
        nodeTree.getControl().setLayoutData(
                new GridData(TREE_WIDTH, TREE_HEIGHT));
        nodeTree.setLabelProvider(getLabelProvider());
        nodeTree.setAutoExpandLevel(2);
        nodeTree.setContentProvider(getContentProvider());
        
        nodeTree.addSelectionChangedListener(new FolderSelectionListener(this));

        nodeTree.getTree().addMouseListener(mouseList);
        nodeTree.getTree().addKeyListener(keyList);
        nodeTree.setInput(ResourcesPlugin.getWorkspace());

        this.getShell().setText(getTitle());
        this.getShell().setImage(
                PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_OBJ_ELEMENT));

        refresh();

        return container;
    }

    protected ILabelProvider getLabelProvider() {
        return WorkbenchLabelProvider
                .getDecoratingWorkbenchLabelProvider();
    }

    protected ITreeContentProvider getContentProvider() {
    	ITreeContentProvider prov = new ITreeContentProvider() {

    		    public void dispose() {

    		    }

    		    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {

    		    }

    		    public Object[] getElements(Object inputElement) {
    		    	 return ResourcesPlugin.getWorkspace().getRoot().getProjects();
    		    }

    		    public Object[] getChildren(Object parentElement) {
    		        if (parentElement instanceof IProject) {
    		            IProject projects = (IProject) parentElement;
    		            try {
    		            	List<Object> mems = new ArrayList<Object>();
    		            	for(IResource ir : projects.members())
    		            		if(ir instanceof IFolder && !((IFolder)ir).isHidden())
    		            			mems.add(ir);
    		                return mems.toArray();
    		            } catch (CoreException e) {
    		                e.printStackTrace();
    		            }
    		        }
    		        if (parentElement instanceof IFolder) {
    		            IFolder ifolder = (IFolder) parentElement;
    		            try {
    		            	List<Object> mems = new ArrayList<Object>();
    		            	for(IResource ir : ifolder.members())
    		            		if(ir instanceof IFolder && !((IFolder)ir).isHidden())
    		            			mems.add(ir);
    		                return mems.toArray();
    		            } catch (CoreException e) {
    		                // TODO Auto-generated catch block
    		                e.printStackTrace();
    		            }
    		        }

    		        return null;
    		    }

    		    public Object getParent(Object element) {
    		        if (element instanceof IProject) {
    		            IProject projects = (IProject) element;
    		            return projects.getParent();
    		        }
    		        if (element instanceof IFolder) {
    		            IFolder folder = (IFolder) element;
    		            return folder.getParent();
    		        }
    		        return null;
    		    }

    		    public boolean hasChildren(Object element) {
    		    	Object[] child = getChildren(element);
    		    	if(child == null)
    		    		return false;
    		        return child.length > 0;
    		    }
		};
        return prov;
    }

    protected String getRootQId() {
        return "/";
    }

    protected Composite createClientContainer(Composite parent, int span) {
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.marginWidth = layout.marginHeight = 10;
        layout.verticalSpacing = 2;
        layout.numColumns = span;
        container.setLayout(layout);
        return container;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        super.createButtonsForButtonBar(parent);
        updateUI();
    }

    public void setCanSelectRoot(boolean canSelectRoot) {
        this.canSelectRoot = canSelectRoot;
    }

    public void select(Object obj) {
        if (obj != null) {
            selectedNode = (IFolder) obj;
            if (nodeTree != null && !nodeTree.getControl().isDisposed()) {
                nodeTree.expandToLevel(selectedNode, 0);
                nodeTree.setSelection(new StructuredSelection(obj), true);
                updateUI();
            }
        }
    }

    private void refresh() {
        if (nodeTree != null && !nodeTree.getControl().isDisposed()) {
            nodeTree.refresh();

            if (selectedNode != null) {
                 select(selectedNode);
            }
        }
    }

    public CreateFileDialog(String extension,String title,Shell shell) {
        super(shell);
        this.extension = extension;
        this.title = title;
    }

    public String getTitle() {
        return title;
    }
}