/*
 * Copyright 2009-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.wizards;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveRegistry;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.ResourceStorage;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.natures.ReqCatalogue;
import com.unitesk.requality.eclipse.storageformatversion.StoredFormatVersioning;
import com.unitesk.requality.eclipse.ui.perspective.ReqEditorPerspectiveFactory;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.NodeType;
import com.unitesk.requality.nodetypes.ReportFolder;
import com.unitesk.requality.nodetypes.Requirement;

public class NewRequalityProjectWizard extends Wizard implements INewWizard {
    private static final String REQUALITY_PERSPECTIVE_PREFIX = "com.unitesk.requality.";
    private NewRequalityProjectPage page;
    private ISelection selection;
    /**
     * Constructor for ReqWizard.
     */
    public NewRequalityProjectWizard() {
        super();
        setNeedsProgressMonitor(true);
    }

    /**
     * Adding the page to the wizard.
     */
    public void addPages() {
        page = new NewRequalityProjectPage(selection);
        addPage(page);
    }

    @Override
    public boolean performFinish() {
        final String projectName = page.getContainerName();
        try {
            IWorkspace workspace = ResourcesPlugin.getWorkspace();
            IProject project = workspace.getRoot().getProject(projectName);
            TreeDB.create(projectName, new ResourceStorage(project));
            List<String> vers = new ArrayList<String>();
        	vers.add(StoredFormatVersioning.getFullVersionByGroupId(TreeNode.NODE_GROUP));
        	StoredFormatVersioning.setFormatsVersions(project, vers);
            TreeDB db = TreesTracker.getTracker().getTree(project);
            // Create nodes
            TreeNode node = db.getRootNode();
            if (node.findChild("Requirements")==null || node.findChild("Types")==null || node.findChild("Documents")==null || node.findChild("Reports")==null) {
                db.startTransaction("Create Project Structure");
                if(node.findChild("Requirements")==null) {
                    node.createChild("Requirements", Requirement.TYPE_NAME);
                }
                if(node.findChild("Types")==null) {
                    node.createChild("Types", NodeType.TYPE_NAME);
                }
                if(node.findChild("Documents")==null) {
                    node.createChild("Documents", DocFolder.TYPE_NAME);
                }
                if(node.findChild("Reports")==null) {
                    node.createChild("Reports", ReportFolder.TYPE_NAME);
                }
                db.commit();
            }
            updatePerspective();
        } catch (WorkbenchException e) {
            e.printStackTrace();
        }
        return true;
    }

    public static IProject performFinishNewElement(String projectName)
            throws CoreException {
        IWorkspace workspace = ResourcesPlugin.getWorkspace();
        IWorkspaceRoot root = workspace.getRoot();
        IProject project = root.getProject(projectName);
        IProjectDescription desc = workspace.newProjectDescription(projectName);
        desc.setNatureIds(new String[] { ReqCatalogue.ID });
        project.create(desc, null);
        project.open(null);
        project.setDefaultCharset("UTF-8", null);
        return project;
    }

    private void updatePerspective() throws WorkbenchException {
        IWorkbenchPage ap = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        if (!ap.getPerspective().getId()
                .startsWith(REQUALITY_PERSPECTIVE_PREFIX)) {
            IPerspectiveRegistry reg = PlatformUI.getWorkbench()
                    .getPerspectiveRegistry();
            IPerspectiveDescriptor perspective = reg
                    .findPerspectiveWithId(ReqEditorPerspectiveFactory.ID);
            ap.setPerspective(perspective);
        }
    }

    /**
     * We will accept the selection in the workbench to see if we can initialize
     * from it.
     * 
     * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        this.selection = selection;
    }
}