package com.unitesk.requality.nodetypes;

import java.io.File;
import java.util.UUID;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.tools.RequalityCLI;

public class TestPurpose extends TreeNode {

    public static final String TYPE_NAME = "TestPurpose";
    public static final String ATTR_EXPECTED_RESULTS = "_expectedResults";
    public static final String ATTR_STATUS = "_status";
    public static final String ATTR_AUTHOR = "_author";
    public static final String ATTR_NAME = "_name";
    public static final String ATTR_TEST_METHOD = "_testMethod";
    public static final String ATTR_TEST_LOCATION = "_testLocation";

    // Constructor for eclipse instancer
    public TestPurpose() {
        super(null, null);
    }
    
    private void fixAttributes() {
    	String oldResults = (String) getAttributeValue("expectedResults","");
    	if(oldResults == null || oldResults.isEmpty()) return;
    	removeAttribute("expectedResults");
    	setExpectedResults(getAttributeValue(ATTR_EXPECTED_RESULTS,"") + oldResults); 	
    }
    
    public TestPurpose(TreeDB db, UUID id) {
        super(db, id);
    }

    public String getTargetRequirementQId() {
        if(getParent()==null) return null;
        return getParent().getQualifiedId();
    }

    public void setTargetRequirementQId(String targetReqQId) {
        if(! targetReqQId.equals(getParent().getQualifiedId())) {
            setParent(getTreeDB().getNode(targetReqQId));
        }
    }

    public String getExpectedResults(String prefix, String postfix, String basePath) {
        String desc = getExpectedResults(false);
        if(desc == null)
        	return desc;
    	return RequalityCLI.getReportDescription(this,
        		getTreeDB().getStorage().getNodeResourcePath(getUUId()) + File.separator,
                desc, prefix, postfix, basePath);
    }
    
    public String getExpectedResults(String basePath) {
        String result = getExpectedResults("<div>", "</div>", basePath);
        return result;
    }
    
    public String getExpectedResults(boolean process) {
        fixAttributes();
        String result = (String) getAttributeValue(ATTR_EXPECTED_RESULTS,"");
        if (process) {
            result = processResourcesPath(result);
        }
        return result;
    }

    public void setExpectedResults(String expectedResults) {
        putAttribute(new Attribute(AttributeType.STRING,
                ATTR_EXPECTED_RESULTS, expectedResults));
    }
    @Override
    public String toString() {
        return getId();
    }
}
