/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.report;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

public class TemplateFile {
    private IReportDataSource dataObject;
    private String templatePath;
    private String outputFile;
    private String dataClass;
    private String reportPath;
    private boolean readOnly;

    public TemplateFile(String reportPath, String templatePath, String outputFile,
            String dataClass, boolean readOnly) {
        super();
        this.reportPath = reportPath;
        this.templatePath = templatePath;
        this.outputFile = outputFile;
        this.dataClass = dataClass;
        this.readOnly = readOnly;
    }

    public String getDataClass() {
        return dataClass;
    }

    public String getTemplatePath() {
        return templatePath;
    }

    public String getOutputFileName() {
        return outputFile;
    }
    
    public boolean getReadOnly() {
        return readOnly;
    }
    
    public IReportDataSource instanceDataObject()
            throws InstantiationException, IllegalAccessException, ClassNotFoundException {
        return instanceDataObject(ClassLoader.getSystemClassLoader());
    }

    public IReportDataSource instanceDataObject(ClassLoader baseClassLoader)
            throws InstantiationException, IllegalAccessException,
            ClassNotFoundException {
        if(dataObject!=null) {
            return dataObject;
        }
        URL classPaths[] = null;
        try {
            classPaths = new URL[] { new File(reportPath + File.separator
                    + "classes").toURI().toURL() };
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }

        URLClassLoader cl = URLClassLoader.newInstance(classPaths, baseClassLoader);
        @SuppressWarnings("unchecked")
        Class<IReportDataSource> ds = (Class<IReportDataSource>) cl
                .loadClass(getDataClass());
        dataObject = ds.newInstance();
        return dataObject;
    }

    public void setDataObject(IReportDataSource dataObject) {
        this.dataObject = dataObject;
    }
}
