package com.unitesk.requality.tests;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;

public class CoverageFromFile implements ICoverageStorageSource {
	String lastErrors = "";
	private CoverageStorage targetStorage;
	HashMap<String, TreeNodeCoverageInfo> coverageInfos;
	Map<String, TestErrorInfo[]> errorInfos;
	private String filename;
	
	@Override
	public void setTargetStorage(CoverageStorage cs){
		targetStorage = cs;
	}
	
	@Override
	public Map<String, TreeNodeCoverageInfo> getCoverage() {
		if(filename == null)
			return null;
		if(coverageInfos!=null || loadCoverageFromFile(filename))
			return coverageInfos;
		return null;
	}
	@Override 
	public String getLastErrors(){
		return lastErrors;
	}
	
	@Override
	public void setSettings(String... setts) {
		if(setts.length<1)
			return;
		filename = setts[0];
	}

	@Override
	public String[] getSettings() {
		return new String[]{filename};
	}
	
	public boolean loadCoverageFromFile(String filename) {
		OperationResult or = loadCoverageFromFileOR(filename);
		this.lastErrors = or.getMessage();
		return or.getSucceedFlag();
	}
	public OperationResult loadCoverageFromFileOR(String filename) {
		String lastErrors = "";
		InputStream is = null;
    	try {
    		URL url = null;
    		try{
    			url = new URL("file:"+File.separator+File.separator+File.separator +filename);
        		is = url.openStream();
    		}catch(IOException ex){
    			url = new URL("file:"+File.separator+File.separator +filename);
        		is = url.openStream();
    		}
    		coverageInfos = new HashMap<String,TreeNodeCoverageInfo>();
    		HashMap<String, List<TestErrorInfo>> tempErrorInfos = 
    				new HashMap<String, List<TestErrorInfo>>();
    		errorInfos = new HashMap<String, TestErrorInfo[]>();
    		DocumentBuilderFactory factory =
    				DocumentBuilderFactory.newInstance();
    		DocumentBuilder builder = factory.newDocumentBuilder();	 
    		Document document =
    				builder.parse(is);
    		NodeList nodeList = document.getDocumentElement().getChildNodes();
    		int errorinc = 0;
    		for (int i = 0; i < nodeList.getLength(); i++) {
    			Node node = nodeList.item(i);
    			if (node instanceof Element) {
    				Element el = (Element)node;
    				//coverage info
    				if(node.getNodeName().toLowerCase().equals("reqcoverage")){
    					TreeNodeCoverageInfo nci;
    					String qid = node.getAttributes().
    				            getNamedItem("qid").getNodeValue();
    					if(targetStorage!=null && targetStorage.getTreeDB() != null)
    					{
    						TreeNode nde = targetStorage.getTreeDB().getNode(qid);
    						if(nde!=null)
    							qid = nde.getQualifiedId();
    					}
    					if (!coverageInfos.containsKey(qid)){
    						nci = new TreeNodeCoverageInfo();
    						nci.setTarget(qid);
    					}else
    						nci = coverageInfos.get(qid);
    					NodeList childNodes = node.getChildNodes();
    					for (int j = 0; j < childNodes.getLength(); j++) {
    						Node cNode = childNodes.item(j);
    						if (cNode instanceof Element) {
    							Element cEl = (Element)cNode;
    							String name = cNode.getNodeName();
    							if (name.toLowerCase().equals("covered_by")){
    								String path = cEl.getAttribute("uri");
    								int cnt = 1;
    								if(cEl.hasAttribute("hits"))
    									cnt = Integer.parseInt(cEl.getAttribute("hits"));
    								//if(cEl.hasAttribute("uri"))
    									//local = Boolean.parseBoolean(cEl.getAttribute("local"));
    								nci.registerTest(path, cnt);
    							}
    						}
    					}
    					if(el.hasAttribute("hits"))
    						nci.setCntAll(Integer.parseInt(el.getAttribute("hits")));
    					coverageInfos.put(nci.getTargetQid(), nci);
    				}else if(node.getNodeName().toLowerCase().equals("error")){
    					String name = "error"+errorinc;
    					if(((Element) node).hasAttribute("name"))
    						name = ((Element) node).getAttribute("name");
    					String path = ((Element) node).getAttribute("uri"), desc = "";
    					NodeList childNodes = node.getChildNodes();
    					List<String> qids = new ArrayList<String>();
    					boolean isPlain = true;
    					for (int j = 0; j < childNodes.getLength(); j++) {
    						Node cNode = childNodes.item(j);
    						if (cNode instanceof Element) {
    							Element cEl = (Element)cNode;
    							String cname = cNode.getNodeName();
    							if (cname.toLowerCase().equals("violates")){
    								qids.add(cEl.getAttribute("qid"));
    							}else if (cname.toLowerCase().equals("description")){
    								if(cEl.hasAttribute("format"))
    									isPlain = cEl.getAttribute("format").toLowerCase().equals("plain");
    								desc = cNode.getTextContent();
    							}
    						}
    					}
    					TestErrorInfo tie = new TestErrorInfo(name, path,
    							isPlain, desc, qids.toArray(new String[0]));
    					if(!tempErrorInfos.containsKey(path))
    						tempErrorInfos.put(path, new ArrayList<TestErrorInfo>());
    					tempErrorInfos.get(path).add(tie);
    				}
    			}
    		}
    		for(String s:tempErrorInfos.keySet())
    			errorInfos.put(s, tempErrorInfos.get(s).toArray(new TestErrorInfo[0]));
    		this.lastErrors=lastErrors;
    		return new OperationResult(lastErrors, true);
    	}
    	catch(IOException ie){
    		lastErrors+=CoreUtils.htmlEscape(ie.toString())+"<br>";
    	}
    	catch(SAXException se){
    		lastErrors+=CoreUtils.htmlEscape(se.toString())+"<br>";
    	} 
    	catch (ParserConfigurationException e) {
    		lastErrors+=CoreUtils.htmlEscape(e.toString())+"<br>";
		}
    	finally{
			try {
				if(is!=null)
					is.close();
			} catch (IOException e) {
				lastErrors+=CoreUtils.htmlEscape(e.toString())+"<br>";
			}
    	}
    	this.lastErrors = lastErrors;
    	return new OperationResult(lastErrors, false);
	}
	@Override
	public Map<String, TestErrorInfo[]> getErrors() {
		if(filename == null)
			return null;
		if(coverageInfos!=null || loadCoverageFromFile(filename))
			return errorInfos;
		return null;
	}

	@Override
	public ICoverageSourceWizard getWizard(TreeDB db) {
		return null;
	}
	
}
